@@ -93,6 +93,9 @@ class Skipped(Event):
     """Test case was skipped for some reason. Reason will be captured in the reason field."""
 
     reason: str = attr.ib()
+    test_plan_id: Optional[str] = attr.ib(default=None)
+    test_suite_id: Optional[str] = attr.ib(default=None)
+    test_case_id: Optional[str] = attr.ib(default=None)
 
 
 @attr.s(slots=True)
