@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import base64
 import datetime
 import logging
@@ -6,6 +8,7 @@ from logging import LogRecord
 from typing import Any, Dict, List, Optional, cast
 
 import attr
+import curlify
 import requests
 
 from .events import Payload
@@ -15,6 +18,13 @@ from .types import Headers
 from .utils import format_exception
 
 
+@attr.s(slots=True, repr=False)
+class Case:
+    text_lines: Optional[List[str]] = attr.ib(default=None)
+    requests_code: Optional[str] = attr.ib(default=None)
+    curl_code: Optional[str] = attr.ib(default=None)
+
+
 @attr.s(slots=True, repr=False)
 class Check:
     """Single check run result."""
@@ -23,6 +33,7 @@ class Check:
     value: Status = attr.ib()
     response: Optional[requests.Response] = attr.ib()
     elapsed: float = attr.ib()
+    example: Optional[Case] = attr.ib(default=None)
     message: Optional[str] = attr.ib(default=None)
     # Failure-specific context
     context: Optional[FailureContext] = attr.ib(default=None)
@@ -57,6 +68,20 @@ class Request:
             body=base64.b64encode(body).decode() if body is not None else body,
         )
 
+    def as_curl_command(self) -> str:
+        """Construct a curl command for a given Request."""
+        prepared_request = requests.PreparedRequest()
+        prepared_request.prepare(
+            url=self.uri,
+            method=self.method,
+            headers=self.headers,
+            data=base64.b64decode(self.body.encode()).decode()
+            if self.body is not None
+            else self.body,
+        )
+
+        return curlify.to_curl(prepared_request)
+
 
 def serialize_payload(payload: bytes) -> str:
     return base64.b64encode(payload).decode()
@@ -136,10 +161,21 @@ class TestResult:
     interactions: List[Interaction] = attr.ib(factory=list)
     logs: List[LogRecord] = attr.ib(factory=list)
     is_errored: bool = attr.ib(default=False)
+    summary: Optional[str] = attr.ib(default=None)
     # To show a proper reproduction code if an error happens and there is no way to get actual headers that were
     # sent over the network. Or there could be no actual requests at all
     overridden_headers: Optional[Dict[str, Any]] = attr.ib(default=None)
 
+    def __add__(self, other: TestResult) -> TestResult:
+        return TestResult(
+            checks=self.checks + other.checks,
+            errors=self.errors + other.errors,
+            interactions=self.interactions + other.interactions,
+            logs=self.logs + other.logs,
+            is_errored=self.is_errored or other.is_errored,
+            summary=self.summary,
+        )
+
     def mark_errored(self) -> None:
         self.is_errored = True
 
@@ -280,6 +316,7 @@ class InitializedPayload(Payload):
 class BeforeTestSuiteExecutionPayload(Payload):
     name: str = attr.ib()
     test_suite_id: Optional[str] = attr.ib(default=None)
+    modules: List[str] = attr.ib(factory=list)
 
 
 @attr.s(slots=True)
@@ -303,6 +340,8 @@ class BeforeTestExecutionPayload(Payload):
     # The current level of recursion during stateful testing
     recursion_level: int = attr.ib(default=0)
 
+    module: Optional[str] = attr.ib(default=None)
+
 
 @attr.s(slots=True)
 class AfterTestExecutionPayload(Payload):
@@ -317,6 +356,28 @@ class AfterTestExecutionPayload(Payload):
     test_suite_id: Optional[str] = attr.ib(default=None)
     thread_id: int = attr.ib(factory=threading.get_ident)
 
+    def __add__(self, other: AfterTestExecutionPayload) -> AfterTestExecutionPayload:
+        result = None
+        if not self.result:
+            result = other.result
+        elif not other.result:
+            result = self.result
+        else:
+            result = self.result + other.result
+
+        return AfterTestExecutionPayload(
+            test_case_id=self.test_case_id,
+            name=self.name,
+            method=self.method,
+            path=self.path,
+            relative_path=self.path,
+            status=self.status + other.status,
+            elapsed_time=self.elapsed_time + other.elapsed_time,
+            result=result,
+            test_suite_id=self.test_suite_id,
+            thread_id=self.thread_id,
+        )
+
 
 @attr.s(slots=True)
 class AfterStepExecutionPayload(Payload):
